/******/ (() => { // webpackBootstrap
/******/ 	"use strict";
var __webpack_exports__ = {};

// Initialize content script
console.log("🔄 Automation sequence content script loading...");
// Constants for reactive timing
const TIMING = {
    // Maximum time to wait for any action
    MAX_TIMEOUT: 8000,
    // Time to wait for DOM mutations to settle
    MUTATION_SETTLE: 300,
    // Minimum time between actions (for stability)
    MIN_ACTION_SPACING: 200,
    // Action execution minimums (to ensure stability)
    ACTION_MINIMUMS: {
        type: 1000, // Minimum time to ensure typing completes
        click: 500, // Minimum time for click to register
        navigate: 500, // Minimum time for navigation to start
        findVal: 300, // Minimum time to find values
    },
};
// Original timing constants (kept for compatibility)
const INITIAL_DELAY = 1000;
const ACTION_TIMEOUT = 15000;
const VERIFICATION_INTERVAL = 500;
// Track script state
let currentAction = null;
let actionInProgress = false;
let lastActionTime = Date.now();
let mutationObserver = null;
let sequenceState = null;
// Setup mutation observer to track DOM stability with improved efficiency
function setupMutationObserver() {
    return new Promise((resolve) => {
        let timeout;
        let mutationCount = 0;
        const MAX_MUTATIONS = 50; // Cap for mutation count
        const lastMutation = Date.now();
        mutationObserver = new MutationObserver(() => {
            mutationCount++;
            // If we exceed mutation threshold, consider DOM unstable and resolve
            if (mutationCount > MAX_MUTATIONS) {
                mutationObserver?.disconnect();
                resolve();
                return;
            }
            if (timeout)
                clearTimeout(timeout);
            timeout = setTimeout(() => {
                // If no mutations for MUTATION_SETTLE time, consider DOM stable
                if (Date.now() - lastMutation >= TIMING.MUTATION_SETTLE) {
                    mutationObserver?.disconnect();
                    resolve();
                }
            }, TIMING.MUTATION_SETTLE);
        });
        // Observe only specific mutation types that matter
        mutationObserver.observe(document.documentElement, {
            childList: true,
            subtree: true,
            attributes: true,
            attributeFilter: ["class", "style", "disabled"], // Only watch relevant attributes
            characterData: false, // Don't watch text changes
        });
        // Resolve anyway after max timeout
        setTimeout(() => {
            mutationObserver?.disconnect();
            resolve();
        }, TIMING.MAX_TIMEOUT);
    });
}
// Enhanced waitForDOMStability that works with original functions
async function waitForDOMStability(actionType) {
    if (mutationObserver) {
        mutationObserver.disconnect();
    }
    // Only setup mutation observer for actions that need it
    if (actionType === "navigate" || actionType === "click") {
        await setupMutationObserver();
    }
    // Ensure minimum time between actions with dynamic spacing
    const timeSinceLastAction = Date.now() - lastActionTime;
    const minSpacing = actionType
        ? TIMING.MIN_ACTION_SPACING +
            (TIMING.ACTION_MINIMUMS[actionType] || 0)
        : TIMING.MIN_ACTION_SPACING;
    if (timeSinceLastAction < minSpacing) {
        await new Promise((r) => setTimeout(r, minSpacing - timeSinceLastAction));
    }
}
// Handle document ready state
if (document.readyState === "loading") {
    document.addEventListener("DOMContentLoaded", handleDocumentReady);
}
else {
    handleDocumentReady();
}
async function handleDocumentReady() {
    console.log("✅ Document ready");
    await waitForDOMStability();
}
// Action verification helper
async function verifyActionComplete(details) {
    switch (details.action) {
        case "navigate":
            // For navigation, check if we're at the target URL
            return window.location.href.includes(details.URL);
        case "clickButton":
            // For clicks, check if the element is still in the DOM
            const selector = buildSelector(details);
            const element = document.querySelector(selector);
            return !element || element.getAttribute("disabled") === "true";
        case "type":
            try {
                // Get expected values from storage
                const { userSelector, passSelector, slug } = details;
                const userData = await getStorageData([`UserKeys_${slug}`]);
                const passData = await getStorageData([`PassKeys_${slug}`]);
                const expectedUser = userData[`UserKeys_${slug}`];
                const expectedPass = passData[`PassKeys_${slug}`];
                // Get actual input values
                const userInput = document.querySelector(userSelector);
                const passInput = document.querySelector(passSelector);
                if (!userInput?.value || !passInput?.value) {
                    console.log("⚠️ Input fields are empty or not found");
                    return false;
                }
                // Strict value comparison
                const userMatch = userInput.value === expectedUser;
                const passMatch = passInput.value === expectedPass;
                if (!userMatch || !passMatch) {
                    console.log("⚠️ Input verification failed - attempting one more fill");
                    // Clear the fields first
                    userInput.value = "";
                    passInput.value = "";
                    // Small delay before refilling
                    await new Promise((r) => setTimeout(r, 500));
                    // Refill the fields
                    await inputCredentials(userSelector, expectedUser, passSelector, expectedPass);
                    // Wait a moment for the fill to complete
                    await new Promise((r) => setTimeout(r, 500));
                    // Check again
                    const userMatchRetry = userInput.value === expectedUser;
                    const passMatchRetry = passInput.value === expectedPass;
                    if (!userMatchRetry || !passMatchRetry) {
                        console.log("⚠️ Input verification failed after retry:");
                        if (!userMatchRetry)
                            console.log("- Username still doesn't match expected value");
                        if (!passMatchRetry)
                            console.log("- Password still doesn't match expected value");
                        return false;
                    }
                    console.log("✅ Input values verified after retry");
                    return true;
                }
                console.log("✅ Input values verified - exact match with stored values");
                return true;
            }
            catch (error) {
                console.log("⚠️ Error during type verification:", error);
                return false;
            }
        default:
            return true;
    }
}
// Message handler with improved timing while preserving original behavior
chrome.runtime.onMessage.addListener((message, sender, sendResponse) => {
    (async () => {
        console.log("📥 Received message:", message);
        if (message.action === "executeAction") {
            // Quick response to acknowledge receipt
            sendResponse({ status: "received" });
            if (actionInProgress) {
                console.log("⚠️ Action already in progress:", currentAction);
                chrome.runtime.sendMessage({
                    type: "ACTION_STATUS",
                    status: "rejected",
                    success: false,
                    error: "Another action is already in progress",
                    currentAction,
                    lastActionTime,
                    actionId: message.actionDetails?.actionId,
                });
                return;
            }
            const actionDetails = message.actionDetails || message.action;
            console.log("⏱️ Action duration:", actionDetails.duration || "default");
            console.log("Processing action:", actionDetails);
            currentAction = actionDetails.action;
            actionInProgress = true;
            lastActionTime = Date.now();
            try {
                const actionResult = await executeAction(actionDetails);
                currentAction = null;
                actionInProgress = false;
                lastActionTime = Date.now();
                // Send completion status with action ID
                chrome.runtime.sendMessage({
                    type: "ACTION_STATUS",
                    ...actionResult,
                    actionId: actionDetails.actionId,
                    action: actionDetails.action,
                    completedAt: Date.now(),
                });
            }
            catch (error) {
                console.error("Action failed:", error);
                currentAction = null;
                actionInProgress = false;
                lastActionTime = Date.now();
                chrome.runtime.sendMessage({
                    type: "ACTION_STATUS",
                    status: "completed",
                    success: false,
                    error: String(error),
                    actionId: actionDetails.actionId,
                    action: actionDetails.action,
                    completedAt: Date.now(),
                });
            }
        }
        else if (message.action === "ping" ||
            message.action === "contentScriptReady") {
            sendResponse({
                ready: !actionInProgress,
                currentAction,
                actionInProgress,
                lastActionTime,
                timeSinceLastAction: Date.now() - lastActionTime,
            });
        }
    })();
    return true; // Keep the message channel open
});
// Add new action execution function
async function executeFindValAction(actionDetails) {
    try {
        const result = await universalFindText(actionDetails);
        return {
            status: "completed",
            success: result !== null,
            error: result === null ? "No value found" : null,
            data: result,
        };
    }
    catch (error) {
        return {
            status: "completed",
            success: false,
            error: String(error),
        };
    }
}
// Execute action with stability checks
async function executeAction(actionDetails) {
    const startTime = Date.now();
    const actionType = actionDetails.action;
    try {
        let result;
        // Wait for DOM stability if needed
        if (actionType === "navigate" || actionType === "click") {
            await waitForDOMStability(actionType);
        }
        // Execute the action
        switch (actionType) {
            case "type":
                result = await executeTypeAction(actionDetails);
                break;
            case "clickButton":
                result = await executeClickAction(actionDetails);
                break;
            case "navigate":
                result = await executeNavigateAction(actionDetails);
                break;
            case "findVal":
                result = await executeFindValAction(actionDetails);
                break;
            case "switchTab":
                result = await executeSwitchTabAction(actionDetails);
                break;
            case "sequenceComplete":
                result = {
                    status: "completed",
                    success: true,
                    error: null,
                    data: { message: "Sequence completed successfully" },
                };
                break;
            default:
                throw new Error(`Unknown action type: ${actionType}`);
        }
        // Ensure minimum stability time
        const minStabilityTime = actionType in TIMING.ACTION_MINIMUMS
            ? TIMING.ACTION_MINIMUMS[actionType]
            : 200;
        const elapsed = Date.now() - startTime;
        if (elapsed < minStabilityTime) {
            await new Promise((r) => setTimeout(r, minStabilityTime - elapsed));
        }
        return result;
    }
    catch (error) {
        console.error(`Action ${actionType} failed:`, error);
        throw error;
    }
}
// Helper function to build selector
function buildSelector(options) {
    const { element, attribute, selectorText } = options;
    if (element && attribute && selectorText) {
        return `${element}[${attribute}*="${selectorText}"]`;
    }
    else if (element && attribute) {
        return `${element}[${attribute}]`;
    }
    else if (element) {
        return element;
    }
    return "*";
}
// Keep utility functions that are used by executeAction
async function universalClicker(options) {
    console.log("🔍 Starting click operation with options:", options);
    const { accountId, element, textContents, attribute, selectorText, parentElement, parentAttribute, parentSelectorText, } = options;
    // Build and log the selector
    function buildSelector() {
        let selector = "";
        if (element && attribute && selectorText) {
            selector = `${element}[${attribute}*="${selectorText}"]`;
        }
        else if (element && attribute) {
            selector = `${element}[${attribute}]`;
        }
        else if (element) {
            selector = element;
        }
        else {
            selector = "*";
        }
        console.log("🔍 Built selector:", selector);
        return selector;
    }
    // Check if textContents should be replaced with the UserKeys value
    const finalTextContents = await replaceTextContentsIfEmail(accountId, textContents);
    console.log("🔍 Using text contents:", finalTextContents);
    // Simulate human-like click by triggering mouse events
    async function simulateHumanClick(element) {
        console.log("🖱️ Attempting to click element:", element.outerHTML);
        try {
            const rect = element.getBoundingClientRect();
            const x = rect.left + rect.width / 2;
            const y = rect.top + rect.height / 2;
            console.log("🖱️ Click coordinates:", { x, y });
            // Simulate mouse movement to the element
            const moveEvents = [
                new MouseEvent("mousemove", {
                    bubbles: true,
                    cancelable: true,
                    view: window,
                    clientX: x - 10, // Start slightly off the target
                    clientY: y - 10,
                }),
                new MouseEvent("mousemove", {
                    bubbles: true,
                    cancelable: true,
                    view: window,
                    clientX: x, // Move to the target
                    clientY: y,
                }),
            ];
            // Dispatch move events to the element
            for (const event of moveEvents) {
                element.dispatchEvent(event);
            }
            // Dispatch click events
            const events = [
                new MouseEvent("mousedown", {
                    bubbles: true,
                    cancelable: true,
                    view: window,
                    clientX: x,
                    clientY: y,
                    button: 0,
                }),
                new MouseEvent("mouseup", {
                    bubbles: true,
                    cancelable: true,
                    view: window,
                    clientX: x,
                    clientY: y,
                    button: 0,
                }),
                new MouseEvent("click", {
                    bubbles: true,
                    cancelable: true,
                    view: window,
                    clientX: x,
                    clientY: y,
                    button: 0,
                }),
            ];
            // Dispatch events to the element
            for (const event of events) {
                element.dispatchEvent(event);
            }
            console.log("✅ Click simulation completed");
            return true;
        }
        catch (error) {
            console.error("❌ Error during click simulation:", error);
            return false;
        }
    }
    async function clickElements(elements) {
        console.log("🔍 Found elements to click:", elements.length);
        let clickSuccess = false;
        for (const el of elements) {
            if (await simulateHumanClick(el)) {
                clickSuccess = true;
            }
        }
        return clickSuccess;
    }
    function findElementsWithParent(parentSelector, childSelector) {
        console.log("🔍 Looking for elements with parent", {
            parentSelector,
            childSelector,
        });
        const parents = document.querySelectorAll(parentSelector);
        console.log("🔍 Found parents:", parents.length);
        let foundElements = [];
        parents.forEach((parentEl) => {
            const childElements = parentEl.querySelectorAll(childSelector);
            console.log("🔍 Found children in parent:", childElements.length);
            // Collect each found child element
            foundElements.push(...Array.from(childElements));
            const filteredElements = filterElementsByTextContent(Array.from(childElements));
            console.log("🔍 Filtered elements:", filteredElements.length);
            if (filteredElements.length > 0) {
                clickElements(filteredElements);
            }
        });
        return foundElements;
    }
    function filterElementsByTextContent(elements) {
        if (finalTextContents) {
            console.log("🔍 Filtering by text content:", finalTextContents);
            return elements.filter((el) => {
                const matches = el.textContent
                    ?.trim()
                    .toLowerCase()
                    .includes(finalTextContents.trim().toLowerCase());
                if (matches) {
                    console.log("✅ Found matching element:", el.outerHTML);
                }
                return matches;
            });
        }
        return elements;
    }
    const childSelector = buildSelector();
    let clickSuccess = false;
    if (parentElement && parentAttribute && parentSelectorText) {
        const parentSelector = `${parentElement}[${parentAttribute}*="${parentSelectorText}"]`;
        console.log("🔍 Using parent selector:", parentSelector);
        const foundElements = findElementsWithParent(parentSelector, childSelector);
        return foundElements;
    }
    else {
        const elements = document.querySelectorAll(childSelector);
        console.log("🔍 Found elements:", elements.length);
        const foundElements = Array.from(elements);
        const filteredElements = filterElementsByTextContent(foundElements);
        console.log("🔍 Filtered elements:", filteredElements.length);
        if (filteredElements.length > 0) {
            await clickElements(filteredElements);
        }
        else {
            console.log("⚠️ No elements found with the specified criteria");
        }
        return foundElements;
    }
}
// Function to replace text contents with UserKeys value in case of Google or FB Auth
async function replaceTextContentsIfEmail(accountId, textContents) {
    if (textContents === "UserKeys") {
        console.log(`Replacing textContents with UserKeys for account ${accountId}`);
        console.log("textContents", textContents);
        const keys = await getStorageData([`UserKeys_${accountId}`]);
        const userKey = keys[`UserKeys_${accountId}`];
        console.log("userKey", userKey);
        if (userKey) {
            return userKey;
        }
        else {
            console.error(`No UserKeys found for account ${accountId}`);
            return null;
        }
    }
    return textContents;
}
// Add helper function to check element visibility and interactability
function isElementInteractable(element) {
    if (!(element instanceof HTMLElement))
        return false;
    const style = window.getComputedStyle(element);
    const rect = element.getBoundingClientRect();
    // Check basic visibility
    if (style.display === "none" ||
        style.visibility === "hidden" ||
        style.opacity === "0" ||
        rect.width === 0 ||
        rect.height === 0) {
        return false;
    }
    // Check if element is disabled or readonly
    if (element instanceof HTMLInputElement ||
        element instanceof HTMLTextAreaElement) {
        if (element.disabled || element.readOnly) {
            return false;
        }
    }
    // Check if element is behind other elements
    const elementAtPoint = document.elementFromPoint(rect.left + rect.width / 2, rect.top + rect.height / 2);
    if (!element.contains(elementAtPoint) && !elementAtPoint?.contains(element)) {
        return false;
    }
    return true;
}
// Original typing function restored
function simulateTypingElement(element, text) {
    return new Promise(async (resolve, reject) => {
        if (!element) {
            return reject(new Error("Element not found"));
        }
        // Cast element to HTMLElement first to check its type
        if (!(element instanceof HTMLElement)) {
            return reject(new Error("Element is not an HTML element"));
        }
        // Now check if it's an input or textarea
        if (!(element instanceof HTMLInputElement ||
            element instanceof HTMLTextAreaElement)) {
            return reject(new Error("Element is not an input or textarea"));
        }
        const inputElement = element;
        // Ensure element is visible and interactable
        const rect = inputElement.getBoundingClientRect();
        if (rect.width === 0 || rect.height === 0) {
            return reject(new Error("Element is not visible"));
        }
        try {
            inputElement.focus();
            inputElement.click();
            inputElement.value = "";
            const characters = text.split("");
            const getRandomDelay = () => Math.floor(Math.random() * 100) + 50;
            // Type each character sequentially
            for (const char of characters) {
                // Add the character
                inputElement.value += char;
                // Dispatch events for this character
                const events = [
                    new Event("input", { bubbles: true }),
                    new KeyboardEvent("keydown", {
                        bubbles: true,
                        key: char,
                        code: `Key${char.toUpperCase()}`,
                    }),
                    new KeyboardEvent("keypress", {
                        bubbles: true,
                        key: char,
                        code: `Key${char.toUpperCase()}`,
                    }),
                    new KeyboardEvent("keyup", {
                        bubbles: true,
                        key: char,
                        code: `Key${char.toUpperCase()}`,
                    }),
                ];
                // Dispatch all events for this character
                for (const event of events) {
                    try {
                        inputElement.dispatchEvent(event);
                    }
                    catch (e) {
                        console.warn(`⚠️ Failed to dispatch ${event.type} event:`, e);
                    }
                }
                // Wait before typing next character
                await new Promise((resolve) => setTimeout(resolve, getRandomDelay()));
            }
            // Final change event after all characters are typed
            inputElement.dispatchEvent(new Event("change", { bubbles: true }));
            console.log(`✅ Finished typing into element:`, inputElement);
            resolve(true);
        }
        catch (error) {
            //console.error("❌ Error during typing simulation:", error);
            reject(error);
        }
    });
}
// Modify simulateTyping to use the original approach first
async function simulateTyping(selector, text) {
    console.log(`🔍 Looking for input element: ${selector}`);
    // Try original direct selector first
    const element = document.querySelector(selector);
    if (element && isElementInteractable(element)) {
        try {
            await simulateTypingElement(element, text);
            return true;
        }
        catch (error) {
            console.warn("⚠️ Original typing method failed:", error);
        }
    }
    // If original method fails, try alternative approaches
    const elements = Array.from(document.querySelectorAll(selector));
    if (elements.length === 0) {
        console.log("⚠️ No elements found for selector:", selector);
        return false;
    }
    console.log(`Found ${elements.length} potential elements`);
    // Score each element based on various factors
    const elementScores = elements.map((element) => {
        if (!isElementInteractable(element))
            return { element, score: -1 };
        let score = 0;
        // Prefer elements that are inputs or textareas
        if (element instanceof HTMLInputElement ||
            element instanceof HTMLTextAreaElement) {
            score += 5;
        }
        // Prefer elements with matching label text
        const labelElement = element.closest("label") ||
            document.querySelector(`label[for="${element.id}"]`);
        if (labelElement) {
            const labelText = labelElement.textContent?.toLowerCase() || "";
            if (labelText.includes("username") ||
                labelText.includes("email") ||
                labelText.includes("password")) {
                score += 3;
            }
        }
        // Prefer elements with relevant placeholder or aria-label
        const placeholder = element.getAttribute("placeholder")?.toLowerCase() || "";
        const ariaLabel = element.getAttribute("aria-label")?.toLowerCase() || "";
        if (placeholder.includes("username") ||
            placeholder.includes("email") ||
            placeholder.includes("password") ||
            ariaLabel.includes("username") ||
            ariaLabel.includes("email") ||
            ariaLabel.includes("password")) {
            score += 2;
        }
        // Prefer elements within a form
        if (element.closest("form")) {
            score += 1;
        }
        return { element, score };
    });
    // Sort by score and get the best match
    const bestMatch = elementScores
        .filter(({ score }) => score >= 0)
        .sort((a, b) => b.score - a.score)[0];
    if (!bestMatch) {
        console.log("⚠️ No suitable input element found");
        return false;
    }
    console.log(`✅ Selected best matching element with score ${bestMatch.score}:`, bestMatch.element);
    try {
        await simulateTypingElement(bestMatch.element, text);
        console.log(`✅ Successfully typed into selected element`);
        return true;
    }
    catch (error) {
        console.error("❌ Error during typing:", error);
        return false;
    }
}
// Enhance the inputCredentials function with better error handling and retries
async function inputCredentials(userSelector, userText, passSelector, passText) {
    console.log("🔄 Starting credential input");
    try {
        // Type username first
        console.log("👤 Attempting to input username");
        const userSuccess = await simulateTyping(userSelector, userText);
        if (!userSuccess) {
            throw new Error("Failed to type username");
        }
        // Wait between username and password
        await new Promise((r) => setTimeout(r, 500));
        // Type password
        console.log("🔑 Attempting to input password");
        const passSuccess = await simulateTyping(passSelector, passText);
        if (!passSuccess) {
            throw new Error("Failed to type password");
        }
        // Wait for inputs to settle
        await new Promise((r) => setTimeout(r, 500));
        return true;
    }
    catch (error) {
        // console.error("❌ Error during credential input:", error);
        return false;
    }
}
//
// Utility functions Below
//
async function executeClickAction(actionDetails) {
    const clickResult = await universalClicker(actionDetails);
    return {
        status: "completed",
        success: clickResult.length > 0,
        error: clickResult.length === 0 ? "No elements found to click" : null,
    };
}
async function executeNavigateAction(actionDetails) {
    try {
        if (!actionDetails.URL || actionDetails.URL === "undefined") {
            throw new Error("Invalid URL provided: " + actionDetails.URL);
        }
        const startUrl = window.location.href;
        console.log(`🌐 Navigating from ${startUrl} to ${actionDetails.URL}`);
        window.location.href = actionDetails.URL;
        return {
            status: "completed",
            success: true,
            error: null,
        };
    }
    catch (error) {
        return {
            status: "completed",
            success: false,
            error: String(error),
        };
    }
}
// Individual action execution functions
async function executeTypeAction(actionDetails) {
    try {
        const { userSelector, passSelector, slug } = actionDetails;
        // Get credentials in parallel
        const [passData, userData] = await Promise.all([
            getStorageData([`PassKeys_${slug}`]),
            getStorageData([`UserKeys_${slug}`]),
        ]);
        const passText = passData[`PassKeys_${slug}`];
        const userText = userData[`UserKeys_${slug}`];
        if (!userText || !passText) {
            throw new Error(`Missing credentials for ${slug}`);
        }
        // Single typing attempt
        const typeResult = await inputCredentials(userSelector, userText, passSelector, passText);
        // Check if values match or force set them
        const userInput = document.querySelector(userSelector);
        const passInput = document.querySelector(passSelector);
        if (!userInput || !passInput) {
            return {
                status: "completed",
                success: false,
                error: "Input fields not found",
            };
        }
        // Check if trimmed values match
        const userMatches = userInput.value.trim() === userText.trim();
        const passMatches = passInput.value.trim() === passText.trim();
        // If values don't match, force set them
        if (!userMatches) {
            console.log("Force setting username value");
            userInput.value = userText;
            userInput.dispatchEvent(new Event("input", { bubbles: true }));
            userInput.dispatchEvent(new Event("change", { bubbles: true }));
        }
        if (!passMatches) {
            console.log("Force setting password value");
            passInput.value = passText;
            passInput.dispatchEvent(new Event("input", { bubbles: true }));
            passInput.dispatchEvent(new Event("change", { bubbles: true }));
        }
        return {
            status: "completed",
            success: true,
            error: null,
        };
    }
    catch (error) {
        return {
            status: "completed",
            success: false,
            error: String(error),
        };
    }
}
function getStorageData(keys) {
    return new Promise((resolve, reject) => {
        chrome.storage.local.get(keys, (result) => {
            if (chrome.runtime.lastError) {
                reject(chrome.runtime.lastError.message);
            }
            else {
                resolve(result);
            }
        });
    });
}
// Compare and update value in storage
async function compareAndUpdateValue(newValue, accountId) {
    try {
        if (!accountId) {
            console.log("⚠️ Missing accountId in compareAndUpdateValue");
            return false;
        }
        if (newValue === undefined) {
            console.log(`⚠️ Invalid value (undefined) for ${accountId}`);
            return false;
        }
        console.log(`🔄 Sending value update for ${accountId} - newValue: ${newValue}`);
        return new Promise((resolve) => {
            chrome.runtime.sendMessage({
                action: "UPDATE_VALUE",
                accountId,
                value: newValue,
            }, (response) => {
                if (chrome.runtime.lastError) {
                    console.warn(`⚠️ Error updating value for ${accountId}:`, chrome.runtime.lastError);
                    resolve(false);
                    return;
                }
                if (!response?.success) {
                    console.warn(`⚠️ Failed to update value for ${accountId}:`, response?.error || "Unknown error");
                    resolve(false);
                    return;
                }
                console.log(`✅ Successfully updated value for ${accountId} to ${newValue}`);
                resolve(true);
            });
        });
    }
    catch (error) {
        console.warn(`⚠️ Error in compareAndUpdateValue for ${accountId}:`, error);
        return false;
    }
}
// Universal function to find text content based on various selectors
async function universalFindText(options) {
    const { element, attribute, selectorText, textContents, slug, parentElement, parentAttribute, parentSelectorText, } = options;
    let texts = [];
    function extractTexts(elements) {
        elements.forEach((el) => {
            // Special case for Spree - check aria-label
            if (slug === "Spree") {
                const ariaLabel = el.getAttribute("aria-label");
                if (ariaLabel) {
                    texts.push(ariaLabel);
                    return;
                }
            }
            // Default behavior - use textContent
            const text = el.textContent?.trim() || "";
            if (textContents
                ? text.toLowerCase().includes(textContents.toLowerCase())
                : true) {
                texts.push(text);
            }
        });
    }
    function findElementsWithParent(parentSelector, childSelector) {
        document.querySelectorAll(parentSelector).forEach((parentEl) => {
            extractTexts(parentEl.querySelectorAll(childSelector));
        });
    }
    if (parentElement && parentAttribute && parentSelectorText) {
        findElementsWithParent(`${parentElement}[${parentAttribute}*="${parentSelectorText}"]`, element || "*");
    }
    else if (element && attribute && selectorText) {
        extractTexts(document.querySelectorAll(`${element}[${attribute}*="${selectorText}"]`));
    }
    else if (element) {
        extractTexts(document.querySelectorAll(element));
    }
    console.log(`Texts found for ${slug}:`, texts);
    const finalSum = universalFilter(texts, slug);
    await compareAndUpdateValue(finalSum, slug);
    return finalSum;
}
// Filters and finds the smallest non-zero number from the text content
function universalFilter(texts, slug) {
    const divideBy100Slugs = ["FC", "Scrooge"];
    let smallestNumber = Infinity;
    texts.forEach((text) => {
        const cleanedText = text.replace(/,/g, "");
        const numberMatch = cleanedText.match(/\d+(\.\d+)?/);
        if (numberMatch) {
            let number = parseFloat(numberMatch[0]);
            if (divideBy100Slugs.includes(slug)) {
                number /= 100;
            }
            if (number !== 0 && number < smallestNumber) {
                smallestNumber = number;
            }
        }
    });
    if (smallestNumber === Infinity) {
        console.log("No valid non-zero numbers found.");
        return null;
    }
    else if (smallestNumber > 9999) {
        console.log(`Non-zero number too large: ${smallestNumber}`);
        return null;
    }
    else {
        smallestNumber = parseFloat(smallestNumber.toPrecision(5));
        console.log(`Smallest non-zero number found: ${smallestNumber}`);
        return smallestNumber;
    }
}
// Add new switchTab action execution function
async function executeSwitchTabAction(actionDetails) {
    try {
        // Send message to background script to find and switch to the target tab
        return new Promise((resolve) => {
            chrome.runtime.sendMessage({
                action: "SWITCH_TAB",
                urlPattern: "accounts.google.com/o/oauth2/v2/auth/oauthchooseaccount?",
            }, (response) => {
                if (chrome.runtime.lastError) {
                    resolve({
                        status: "completed",
                        success: false,
                        error: String(chrome.runtime.lastError.message),
                    });
                    return;
                }
                if (!response?.success) {
                    resolve({
                        status: "completed",
                        success: false,
                        error: response?.error || "Failed to switch tab",
                    });
                    return;
                }
                resolve({
                    status: "completed",
                    success: true,
                    error: null,
                    data: { tabId: response.tabId },
                });
            });
        });
    }
    catch (error) {
        return {
            status: "completed",
            success: false,
            error: String(error),
        };
    }
}

/******/ })()
;
//# sourceMappingURL=automationSequence.js.map